/*
 * Created on 15 avr. 2004 by the Message Center Team
 *
 */
 
package mc.formgenerator.ggf.xslprocessing;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.xerces.dom.DOMImplementationImpl;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *  Execution of Xalan engine
 * 	@author sempereb
 */
public class XalanExecutor {

	//Path of the xml file
	public String xmlFile;
	
	//Path of the xsl file
	public String xslFile;
	
	//Path of the output file
	public String outputFile;
	
	public Transformer transformer = null;
	
	/**
	 * Default constructor of our class
	 */
	public XalanExecutor(){};




	/**
	 * Other constructor of our class
	 * It will create a XalanExecutor with a defined xsl file
	 * @param theXslFile: path of the xsl file
	 */
	public XalanExecutor(String theXslFile, String userLanguage, String fileXformsBonita) 
		throws TransformerConfigurationException
	{
		//Set path of the xsl file
		xslFile = theXslFile;
		
		//Creation via the factory of an instance of Xalan transfomer
		TransformerFactory tFactory = TransformerFactory.newInstance();
		
		//We read the xsl file
		transformer = tFactory.newTransformer(new StreamSource( xslFile ));
		// we set the language from the user request http
		transformer.setParameter("language", userLanguage);
		// we set the file path
		transformer.setParameter("fileXformsBonita", fileXformsBonita);
	}

	public static XalanExecutor xalanExecutorFormURL(String theXslFile, String userLanguage, String fileXformsBonita) 
		throws TransformerConfigurationException
	{    
	    XalanExecutor n = new XalanExecutor();
	    
	    //Set path of the xsl file
	    n.xslFile = theXslFile;
	
	    //Creation via the factory of an instance of Xalan transfomer
	    TransformerFactory tFactory = TransformerFactory.newInstance();
	    
	    //We read the xsl file
	    n.transformer = tFactory.newTransformer(new StreamSource( theXslFile ));
	    // we set the language from the user request http
	    
	    n.transformer.setParameter("language", userLanguage);	
	    n.transformer.setParameter("fileXformsBonita", fileXformsBonita);    
	    return n;
	};

	


	/**
	 * Other execution with parameters
	 * @param theXmlFile: the source file
	 * @param theXslFile: the file that provides transformation
	 * @param theOutputFile: the file generated
	 * @throws TransformerException
	 * @throws TransformerConfigurationException
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public void XalanExecution(String theXmlFile, String theXslFile, String theOutputFile)
			throws TransformerException, TransformerConfigurationException, 
			FileNotFoundException, IOException{
		
		//Creation via the factory of an instance of Xalan transfomer
		TransformerFactory tFactory = TransformerFactory.newInstance();
	
		//We read the xsl file
		Transformer transformer = tFactory.newTransformer(new StreamSource( theXslFile ));
		
		// we set the language from the user request http
		transformer.setParameter("language","en");

		//Transformation thanks to xalan for the xml file
		transformer.transform(new StreamSource( theXmlFile ), new StreamResult(new FileOutputStream( theOutputFile )));
	}




	/**
	 * Execution of Xalan on files in order to make dynamic XHTML 
	 * Here the output file is a Document instance
	 * @param document: the document to transform
	 * @param document: base uri needed by Chiba
	 * @return a Document that represents the document generated by the xsl engine
	 * @throws TransformerException
	 * @throws TransformerConfigurationException
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public Document XalanExecutionDOM(Document document, String xmlBase)
			throws TransformerException, TransformerConfigurationException, 
			FileNotFoundException, IOException{
		
		//The DOM result
		DOMResult outputDOM = new DOMResult();
		
		//Transformation thanks to xalan for the xml file
		transformer.transform( new DOMSource(document), outputDOM);
		
		Document documentOutput = (Document)outputDOM.getNode();
		
		//Set xml:base to avoid Chiba Exception...
		Element root = documentOutput.getDocumentElement();	
		root.setAttribute("xml:base",xmlBase);
		
		//Update of documentOutput 
		return documentOutput;
	}
	
}
