/*
 * Created on 1 april 2005
 *
 */
package mc.formgenerator.servlets.bonita;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;


/**
 * This class returns the XForm's filename associated with a project or activity.
 * @author cesterbj
 * 
 */
public class RepositoryFacade {

	
	/**
	 * Attribute which stores the repository document
	 */
	private Document xmlDocument;


	/**
	 * Default class constructor
	 */
	public RepositoryFacade(String xmlPath) {	
		DocumentBuilderFactory factory = new org.apache.xerces.jaxp.DocumentBuilderFactoryImpl();
		
		try {
			//get file and parse it to xmlDocument
			File xmlFile = new File(xmlPath);
			DocumentBuilder documentBuilder = factory.newDocumentBuilder();
			this.xmlDocument = documentBuilder.parse(xmlFile);
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
	}



	/**
	 * Returns the path of the XForm file for a project.
	 * @param projectName: the project's name
	 * @return String formpath.
	 * @throws Exception
	 */
	public String getProjectFormPath (String projectName) throws Exception {
		String formPath = null;
		
		//get "project" nodes
		NodeList nodeList = this.xmlDocument.getElementsByTagName("project");
		int size = nodeList.getLength();
		if (size == 0) {
			return null;
		} 
		else {
			//search projectName
			for (int i=0;i<nodeList.getLength();i++){
				Element projectElement = (Element)nodeList.item(i);
				//get project name
				NodeList elementsName = projectElement.getElementsByTagName("projectname");
				Text nameElement = (Text)elementsName.item(0).getFirstChild(); 				
				String name = nameElement.getNodeValue();
	
				if(name.equals(projectName)){
					//if project name has been found in repository, lookup its xform filename
					NodeList elementsXformFile = projectElement.getElementsByTagName("projectxformfile");
					Text xformpathElement = (Text)elementsXformFile.item(0).getFirstChild();
					if(xformpathElement!=null){
						//this project has an xform file associated, then return filename
						String path = xformpathElement.getNodeValue();
						System.out.println("RepositoryFacade ====> Project "+ projectName + " has been found in repository. XForm file: " + path);
						return (path);
					}else return null;
				}
				
			}
		}
		
		return null;
	}
	
	/**
	 * Returns the path of the XForm file for an activity.
	 * @param projectName: the project's name
	 * @param activityName: the activity's name
	 * @return String formpath.
	 * @throws Exception
	 */
	public String getActivityFormPath (String projectName, String activityName) throws Exception {
		String formPath = null;
		
		//get "project" nodes
		NodeList nodeList = this.xmlDocument.getElementsByTagName("project");
		int size = nodeList.getLength();
		if (size == 0) {
			return null;
		} 
		else {	
			//search project name
			for (int i=0;i<nodeList.getLength();i++){
				Element projectElement = (Element)nodeList.item(i);			
				NodeList pElementsName = projectElement.getElementsByTagName("projectname");							
				Text pNameElement = (Text)pElementsName.item(0).getFirstChild();
				String pName = pNameElement.getNodeValue();
				
				if(pName.equals(projectName)){
					//if project name has been found in repository, lookup its xform filename
					NodeList elementsActivities = projectElement.getElementsByTagName("activities");
					Element activitiesElement = (Element)elementsActivities.item(0);
					//get "activity" nodes
					NodeList activitiesNodeList = activitiesElement.getElementsByTagName("activity");
					int activitiesSize = activitiesNodeList.getLength();		
					if (activitiesSize == 0) {
						return null;
					} 
					else {		
						for (int j=0;i<nodeList.getLength();i++){	
							Element activityElement = (Element)nodeList.item(i);
							NodeList aElementsName = activityElement.getElementsByTagName("activityname");
							Text aNameElement = (Text)aElementsName.item(0).getFirstChild(); 
							String aName = aNameElement.getNodeValue();

							if(aName.equals(activityName)){
								//if activity name has been found in repository, lookup its xform filename
								NodeList elementsXformFile = projectElement.getElementsByTagName("activityxformfile");
								Text xformpathElement = (Text)elementsXformFile.item(0).getFirstChild();
								if(xformpathElement!=null){
									//this activity has an xform file associated, then return filename
									String path = xformpathElement.getNodeValue();
									System.out.println("RepositoryFacade ====> Activity "+ activityName +" of project "+ projectName + " has been found in repository. XForm file: " + path);
									return (path);
								}else return null;
							}
						}
					}
				}
			}
		}		
		return null;
	}	

}
