package hero.entity;
import hero.interfaces.Constants;
import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnNodeLocal;
import hero.util.BonitaProjectLocator;
import hero.util.BonitaServiceException;
import hero.util.BonitaServiceValue;
import hero.util.HeroException;
import hero.util.values.BonitaEdgeValue;
import hero.util.values.BonitaProjectValue;

import java.util.*;
import org.apache.log4j.Category;


/*
* 02/01/2002 - 15:24:07
*
* AndEdgeState.java - 
* Copyright (C) 2002 Ecoo Team
* charoy@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

public class AndEdgeState extends EdgeState implements java.io.Serializable {

    private static AndEdgeState nd;

    public AndEdgeState() {
    }

    public static AndEdgeState getInstance() {
	if (nd==null) {
	    nd=new AndEdgeState();
	}
	return nd;
    }

    public int state(BnNodeLocal node) {
	BonitaProjectValue currentProject=null;
	Collection edges = new ArrayList();
	Collection edgesModel = new ArrayList();
	
	try{
		edges = BonitaServiceValue.getNode(node.getBnProject(),node.getName()).getInEdges();
	}catch (Exception e) {e.printStackTrace();}
		
	// Instances consistency (may be some activities are not yet created in the instance)
	String nodeName = node.getName();
	if (isInstance(node.getName()))
		nodeName = getModel(nodeName);
		
	try{
		edgesModel = BonitaServiceValue.getNode(node.getBnProject().getModel().getName(),nodeName).getInEdges();
	}catch (Exception e) {edgesModel=new ArrayList();}
	if (edges.size() != edgesModel.size() && !edgesModel.isEmpty())
		edges = concatEdges(edges, edgesModel);	

	Iterator i = edges.iterator();
	int result=Constants.Nd.ACTIVE;
	while (i.hasNext()) {
		BonitaEdgeValue edge = (BonitaEdgeValue)i.next();		
	    if (edge.getState()==Constants.Ed.INITIAL) {
		result=Constants.Nd.EDGEINITIAL;
		break;
	    }
	    if ((edge.getState() == Constants.Ed.ANTICIPATING))
		result=Constants.Nd.ANTACTIVE;
	    if ((edge.getState() == Constants.Ed.DEAD))
		{
		    result=Constants.Nd.CANCEL;
		    break;
		}	    
	}
	return result;	
    }	   
    
	private String getModel(String instanceName) {
		int i = instanceName.indexOf("_instance");
		return (instanceName.substring(0, i));
	}

	private boolean isInstance(String name) {
		return (name.matches(".*_instance.*"));
	}
	
	private Collection concatEdges(Collection edges, Collection edgesModel) {
		Collection result = new ArrayList();
		Iterator ed = edgesModel.iterator();
		while (ed.hasNext())
		{
			boolean found = false;
			BonitaEdgeValue el = (BonitaEdgeValue)ed.next();
			String nodeName = el.getInNode();
			Iterator instEdges = edges.iterator();
			while(instEdges.hasNext())
			{
				BonitaEdgeValue insEd = (BonitaEdgeValue)instEdges.next();
				String node = insEd.getInNode();
				if (this.isInstance(node))
					node = this.getModel(node);
				if (nodeName.equals(node))
				{
					result.add(insEd);
					found = true;
					break;
				}
			}
			if (!found)
				result.add(el);
		}
		return result;
	}
}











