/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: BonitaDeadline.java,v 1.3 2005/03/18 14:55:05 mvaldes Exp $
 *
--------------------------------------------------------------------------
 */

package hero.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.NotificationFilter;
import javax.management.ObjectName;

import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodeUtil;

public final class BonitaDeadline {
	private static BonitaDeadline bdl = null; // Singleton pattern
	private static ObjectName timer;
	private static MBeanServer server;

	/* replaced by startDeadlines
	public void startDeadline(String nodeName, int p_id) throws HeroException{
		try {
			BnNodeLocalHome lh = BnNodeUtil.getLocalHome();
			BnNodeLocal node = lh.findByName(nodeName,p_id);
			
			String type = null;
			Date lNext=null;
			if (node.getDeadline()!=null)
			{
				type=node.getName() + ":" + node.getBnProject().getName()+":"+node.getDeadline().getTime();
				lNext=node.getDeadline();
				addNotification(lNext, type, node);
			}
			if (node.getRelativeDeadline()!=null)
			{
				type=node.getName() + ":" + node.getBnProject().getName()+":"+node.getRelativeDeadline().getTime();
				lNext=new Date(new Date().getTime()+node.getRelativeDeadline().getTime());
				addNotification(lNext, type, node);
			}						
		} catch (Exception e) {
			e.printStackTrace();
			throw new HeroException(e.getMessage());
		}
	}
*/
	
	public void startDeadlines(String nodeName, String p_id) throws HeroException{
		try {
			BnNodeLocalHome lh = BnNodeUtil.getLocalHome();
			BnNodeLocal node = lh.findByName(nodeName,p_id);
			
			String type = null;
			Date lNext=null;
			
			// For absolute deadline
			if (node.getDeadlines().size()!=0) {
//debug	
System.out.println("startDeadlines : 	Nb of absolute deadlines = " + 	node.getDeadlines().size());
				Collection co = node.getDeadlines();
				Iterator it = co.iterator();
				ArrayList ar = new ArrayList();
				while (it.hasNext()){
					// the co contains Date elements
					Date date = (Date)it.next();
					type=node.getName() + ":" + node.getBnProject().getName()+":"+date.getTime();
					lNext = date;
					// add notification for each deadline in the collection
					addNotification(lNext, type, node);
//debug
System.out.println("startDeadlines for absolute deadlines: lnext = " + lNext.toString() + " type = " + type + " node = " + node.getName());			
				}
			}			
			// For relative deadline
			if (node.getRelativeDeadlines().size()!=0) {
//debug
System.out.println("startDeadlines : 	Nb of relative deadlines = " + 	node.getRelativeDeadlines().size());				
				Collection co = node.getRelativeDeadlines();
				Iterator it = co.iterator();
				ArrayList ar = new ArrayList();
				while (it.hasNext()){
					// the co contains Date elements
					Date date = (Date)it.next();
					type=node.getName() + ":" + node.getBnProject().getName()+":"+date.getTime();
					//create absolute time for the notification
					lNext=new Date(new Date().getTime()+date.getTime());
					// add notification for each deadline in the collection
					addNotification(lNext, type, node);
//debug
System.out.println("startDeadlines for relative deadlines: lnext = " + lNext.toString() + " type = " + type + " node = " + node.getName());			
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new HeroException(e.getMessage());
		}
	}

	
	public void removeNodeNotification(String projectName, String nodeName, long date) throws HeroException {
		try {
			ObjectName timer = new ObjectName("jonas:type=service,name=Timer");
			List list = MBeanServerFactory.findMBeanServer(null);
			MBeanServer server = (MBeanServer) list.iterator().next();
			String type=nodeName + ":" + projectName+":"+date;
			
			server.invoke(timer,"removeNotifications",new Object[] { type },new String[] { "".getClass().getName()});
//debug
System.out.println("RemoveDeadLineNotification for project: "+projectName+", node: "+nodeName + " date: " + date);
		} catch (Exception e) {} // Remove and activity which deadline was already thrown
	}
	
	private void addNotification(Date lNext, String type, BnNodeLocal node) throws Exception
	{		
		Vector ids =(Vector) server.invoke(timer, "getNotificationIDs", new Object[] {type,},
		new String[] {"".getClass().getName()});
		
		if (ids.isEmpty())
		{			
			Integer id =(Integer) server.invoke(timer, "addNotification", new Object[] { type,
				//type
				node.getName(), //message
				"bonita", //user object
				lNext, //delay time
			},
			new String[] {
				"".getClass().getName(),
				"".getClass().getName(),
				"java.lang.Object",
				Date.class.getName()});	
//debug			
System.out.println("Deadline: "+lNext+" was added for project: "+node.getBnProject().getName()+", node: "+node.getName());
		}
	}
	
	public static BonitaDeadline getInstance() throws HeroException{
		if (bdl == null) {
			bdl = new BonitaDeadline();
		}
		return bdl;
	}
	
	public BonitaDeadline() throws HeroException{
		try{
			this.timer = new ObjectName("jonas:type=service,name=Timer");
			ObjectName receiver = new ObjectName("jonas:type=service,name=TimerReceiver");
			List list = MBeanServerFactory.findMBeanServer(null);
			this.server = (MBeanServer) list.iterator().next();
			NotificationFilter filter = new TimerFilter();
			this.server.addNotificationListener(timer, receiver, filter, null);
		}catch(Exception e){
			e.printStackTrace();
			throw new HeroException(e.getMessage());
		}
	}
}
