/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: BonitaProjectLocator.java,v 1.3 2005/03/30 15:19:45 mvaldes Exp $
 *
--------------------------------------------------------------------------
 */

package hero.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;

import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnIterationLocal;
import hero.interfaces.BnNodeHookLocal;
import hero.interfaces.BnNodeInterHookLocal;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodePropertyLocal;
import hero.interfaces.BnProjectHookLocal;
import hero.interfaces.BnProjectInterHookLocal;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectPropertyLocal;
import hero.interfaces.BnRoleLocal;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.Constants;
import hero.util.values.BonitaConfigValue;
import hero.util.values.BonitaEdgeValue;
import hero.util.values.BonitaHookValue;
import hero.util.values.BonitaInterHookValue;
import hero.util.values.BonitaIterationValue;
import hero.util.values.BonitaMapperValue;
import hero.util.values.BonitaNodeValue;
import hero.util.values.BonitaPerformerValue;
import hero.util.values.BonitaProjectValue;
import hero.util.values.BonitaPropertyValue;
import hero.util.values.BonitaRoleValue;
import hero.util.values.BonitaUserValue;

import javax.ejb.FinderException;

public class BonitaProjectLocator {
     
	public static BonitaProjectValue generateProjectValue(String projectName) throws BonitaServiceException
	{	
		BnProjectLocalHome pHome=null;
		BnProjectLocal projectModel=null;
		BonitaProjectValue pv = null;
		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();									
			projectModel = pHome.findByName(projectName);						
		} catch (javax.naming.NamingException ne) {
			throw new BonitaServiceException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {throw new BonitaServiceException(fe.getMessage());}
	
		try{
			pv = getProjectData(projectModel);
		} catch (HeroException he) {
			throw new BonitaServiceException(he.getMessage());
		}
      return pv;
	 }   			
		
    private static synchronized BonitaProjectValue getProjectData(BnProjectLocal projectModel) throws HeroException
	{
		BonitaProjectValue pv = new BonitaProjectValue();
		pv.setName(projectModel.getName());
		pv.setType(projectModel.getType());
		pv.setStatus(projectModel.getStatus());
		
		// Project Iterations		
		Collection iterations = projectModel.getBnIterations();
		Iterator iter = iterations.iterator();
		Collection iterPro = new ArrayList();
		while (iter.hasNext())
		{
			BnIterationLocal it = (BnIterationLocal)iter.next();
			BonitaIterationValue iteration = new BonitaIterationValue();
			iteration.setCondition(it.getCondition());
			iteration.setFromNode(it.getFromNode());
			iteration.setToNode(it.getToNode());
			iterPro.add(iteration);
		}
		pv.setIterations(iterPro);
		
		// Project Nodes
		Collection nodesPro = new ArrayList();
		Collection nodes = projectModel.getBnNodes();
		Iterator nodesI = nodes.iterator();
		while (nodesI.hasNext())
		{
			BnNodeLocal nd = (BnNodeLocal)nodesI.next();
			BonitaNodeValue node = new BonitaNodeValue();
			node.setProjectName(projectModel.getName());
			node.setDescription(nd.getDescription());
			node.setDeadlines(nd.getDeadlines());
			node.setRelativeDeadlines(nd.getRelativeDeadlines());
			node.setCreator(nd.getCreator());
			node.setExecutor(nd.getExecutor());
			node.setName(nd.getName());
			node.setRole(nd.getBnRole().getName());
			node.setType(nd.getType());
			node.setProjectParent(nd.getBnProject().getParent());
			node.setAnticipable(nd.getAnticipable());
			
			// Node inEdges
			Collection inEdges = nd.getInBnEdges();
			Iterator inEdgesI = inEdges.iterator();
			Collection nodeInEdges = new ArrayList();
			while(inEdgesI.hasNext())
			{
				BnEdgeLocal el = (BnEdgeLocal)inEdgesI.next();
				BonitaEdgeValue inEdge = new BonitaEdgeValue();
				inEdge.setName(el.getName());
				inEdge.setCondition(el.getCondition());
				inEdge.setState(el.getState());		
				inEdge.setInNode(el.getInBnNode().getName());
				inEdge.setOutNode(el.getOutBnNode().getName());
				nodeInEdges.add(inEdge);
			}
			node.setInEdges(nodeInEdges);
			
			// Node outEdges			
			Collection outEdges = nd.getOutBnEdges();
			Iterator outEdgesI = outEdges.iterator();
			Collection nodeOutEdges = new ArrayList();
			while(outEdgesI.hasNext())
			{
				BnEdgeLocal el = (BnEdgeLocal)outEdgesI.next();
				BonitaEdgeValue outEdge = new BonitaEdgeValue();
				outEdge.setName(el.getName());
				outEdge.setCondition(el.getCondition());
				outEdge.setState(el.getState());		
				outEdge.setInNode(el.getInBnNode().getName());
				outEdge.setOutNode(el.getOutBnNode().getName());
				nodeOutEdges.add(outEdge);
			}
			node.setOutEdges(nodeOutEdges);
			
			// Node properties
			Collection props = nd.getBnProperties();
			Iterator propsI = props.iterator();
			Collection nodeProps = new ArrayList();
			while(propsI.hasNext())
			{
				BnNodePropertyLocal pl = (BnNodePropertyLocal)propsI.next();
				BonitaPropertyValue property = new BonitaPropertyValue();
				property.setKey(pl.getTheKey());
				property.setValue(pl.getTheValue());
				property.setPropagate(pl.getPropagate());
				//if (pl.getPossibleValues()!=null)
					property.setPossibleValues(pl.getPossibleValues());
				nodeProps.add(property);
			}
			node.setProperties(nodeProps);
			BonitaPerformerValue performer = new BonitaPerformerValue();
			performer.setNodeName(nd.getName());
			if (nd.getBnNodePerformerAssign()!=null)
			{
				performer.setName(nd.getBnNodePerformerAssign().getName());
				performer.setType(nd.getBnNodePerformerAssign().getType());
				performer.setPropertyName(nd.getBnNodePerformerAssign().getPropertyName());
				node.setPerformer(performer);
			}
			
			// Node Hooks
			Collection nodeHooks = projectModel.getBnNode(nd.getName()).getBnHooks();
			Iterator nodeHooksI = nodeHooks.iterator();
			Collection nodeHks = new ArrayList();
			while(nodeHooksI.hasNext())
			{
				BnNodeHookLocal ph = (BnNodeHookLocal)nodeHooksI.next();
				BonitaHookValue hook = new BonitaHookValue();
				hook.setName(ph.getName());
				hook.setEvent(ph.getEvent());
				hook.setType(ph.getType());
				nodeHks.add(hook);				
			}
			node.setHooks(nodeHks);
			
			// Node InterHooks
			Collection nodeInterHooks = projectModel.getBnNode(nd.getName()).getBnInterHooks();
			Iterator nodeInterHooksI = nodeInterHooks.iterator();
			Collection nodeInterHks = new ArrayList();
			while(nodeInterHooksI.hasNext())
			{
				BnNodeInterHookLocal pih = (BnNodeInterHookLocal)nodeInterHooksI.next();
				BonitaInterHookValue interHook = new BonitaInterHookValue();
				interHook.setName(pih.getName());
				interHook.setEvent(pih.getEvent());
				interHook.setType(pih.getType());
				interHook.setScript(pih.getScript());
				nodeInterHks.add(interHook);				
			}
			node.setInterHooks(nodeInterHks);			
			
			nodesPro.add(node);
			
		}				
		pv.setNodes(nodesPro);
		
		// Project Config
		
		BonitaConfigValue cv = new BonitaConfigValue();
		cv.setJms(projectModel.getBnProjectConfig().getJMS());
		cv.setTrace(projectModel.getBnProjectConfig().getTrace());
		cv.setLog(projectModel.getBnProjectConfig().getLog());
		cv.setHistoric(projectModel.getBnProjectConfig().getHistoric());
		cv.setLogLevel(projectModel.getBnProjectConfig().getLogLevel());
		cv.setTraceLevel(projectModel.getBnProjectConfig().getTraceLevel());
		pv.setProjectConfig(cv);
		
		// Project Properties
		Collection projectProps = projectModel.getBnProperties();
		Iterator projectPropsI = projectProps.iterator();
		Collection projectPrs = new ArrayList();
		while(projectPropsI.hasNext())
		{
			BnProjectPropertyLocal pl = (BnProjectPropertyLocal)projectPropsI.next();
			BonitaPropertyValue property = new BonitaPropertyValue();
			property.setKey(pl.getTheKey());
			property.setValue(pl.getTheValue());
			//if (pl.getPossibleValues()!=null)
				property.setPossibleValues(pl.getPossibleValues());
			projectPrs.add(property);
		}
		pv.setProperties(projectPrs);
		
		// Project Hooks
		Collection projectHooks = projectModel.getBnHooks();
		Iterator projectHooksI = projectHooks.iterator();
		Collection projectHks = new ArrayList();
		while(projectHooksI.hasNext())
		{
			BnProjectHookLocal ph = (BnProjectHookLocal)projectHooksI.next();
			BonitaHookValue hook = new BonitaHookValue();
			hook.setName(ph.getName());
			hook.setEvent(ph.getEvent());
			hook.setType(ph.getType());
			projectHks.add(hook);				
		}
		pv.setHooks(projectHks);
		
		// Project InterHooks
		Collection projectInterHooks = projectModel.getBnInterHooks();
		Iterator projectInterHooksI = projectInterHooks.iterator();
		Collection projectInterHks = new ArrayList();
		while(projectInterHooksI.hasNext())
		{
			BnProjectInterHookLocal pih = (BnProjectInterHookLocal)projectInterHooksI.next();
			BonitaInterHookValue interHook = new BonitaInterHookValue();
			interHook.setName(pih.getName());
			interHook.setEvent(pih.getEvent());
			interHook.setType(pih.getType());
			interHook.setScript(pih.getScript());
			projectInterHks.add(interHook);				
		}
		pv.setInterHooks(projectInterHks);
		
		// Project Roles
		Collection roles = projectModel.getBnRoles();
		Iterator rolesI = roles.iterator();
		Collection rolesP = new ArrayList();
		while(rolesI.hasNext())
		{
			BnRoleLocal rl = (BnRoleLocal)rolesI.next();
			BonitaRoleValue role = new BonitaRoleValue();
			role.setName(rl.getName());
			role.setDescription(rl.getDescription());
			
			if (rl.getBnRoleMapper()!=null)
			{
				BonitaMapperValue mapper = new BonitaMapperValue();
				mapper.setRoleName(rl.getName());
				mapper.setName(rl.getBnRoleMapper().getName());
				mapper.setType(rl.getBnRoleMapper().getType());
			
				role.setMapper(mapper);
			}
			
			rolesP.add(role);
		}		
		pv.setRoles(rolesP);
		
		// Project
		
		Collection users = projectModel.getBnUsers();
		Iterator usersI = users.iterator();
		Collection usersP = new ArrayList();
		while(usersI.hasNext())
		{
			BnUserLocal ul = (BnUserLocal)usersI.next();
			BonitaUserValue user = new BonitaUserValue();
			user.setName(ul.getName());
			Collection userRoles = getUserRolesInProjectNames(projectModel, ul.getName());
						
			Iterator uRolesI = userRoles.iterator();
			Collection uRolesP = new ArrayList();
			while(uRolesI.hasNext())
			{
				String roleName = (String)uRolesI.next();
				uRolesP.add(roleName);
			}
			user.setRoles(uRolesP);
			usersP.add(user);
		}
		pv.setUsers(usersP);		
		return pv;
	}
    
    private static Collection getUserRolesInProjectNames(BnProjectLocal mProject, String userName)
	throws HeroException {		
		BnUserLocalHome uHome;
		BnUserLocal user;
		ArrayList result = new ArrayList();
		try {
			uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			user = uHome.findByName(userName);
		} catch (FinderException f) {
			throw new HeroException("FinderException");
		}
		Collection userRoles = user.getBnRoles();
		Collection projectRoles = mProject.getBnRoles();
		for (Iterator i = userRoles.iterator(); i.hasNext();) {
			BnRoleLocal ur = (BnRoleLocal) i.next();
			for (Iterator j = projectRoles.iterator(); j.hasNext();) {
				BnRoleLocal up = (BnRoleLocal) j.next();
				if (ur.getId().equals(up.getId()))
					result.add(ur.getBnRoleValue().getName());
			}
		}
		return result;
	}
}
