/*

*
* TimerReceiver.java - 
* Copyright (C) 2003 Ecoo Team
* valdes@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package hero.util;
import java.util.Collection;
import java.util.Iterator;

import hero.interfaces.BnNodeHookLocal;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectUtil;
import hero.interfaces.UserSessionUtil;
import hero.util.values.BonitaHookValue;
import hero.util.values.BonitaNodeValue;
import hero.util.values.BonitaProjectValue;

import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.InstanceAlreadyExistsException;
import javax.management.MBeanRegistrationException;
import javax.management.NotCompliantMBeanException;
import javax.management.MalformedObjectNameException;
import javax.management.NotificationListener;
import javax.management.Notification;
import javax.naming.Context;
import org.objectweb.jonas.service.ServiceException;

//import objectweb
import org.objectweb.jonas.jmx.JmxService;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceManager;

public class TimerReceiver extends AbsServiceImpl implements TimerReceiverMBean, NotificationListener {

	private MBeanServer mbeanServer = null;
	public final static String SERVICE_NAME = "jonas:type=service,name=TimerReceiver";
	private static final UserSessionUtil EngineSessionUtil = null;

	public void doInit(Context ctx) throws ServiceException {
		// Get the JMX Server via JMX Service

	           try {
	           	JmxService jmx = (JmxService) ServiceManager.getInstance().getJmxService();
			     mbeanServer = jmx.getJmxServer();
		   } catch (Exception e) {
			//the JMX service may not be started
			throw new ServiceException("JMX Server is not started...");
		   } 			 	      
	}
	
	public void doStart() throws ServiceException {
		try {
			// Register MailService MBean : MailServiceImplMBean
			mbeanServer.registerMBean(this,new ObjectName(SERVICE_NAME));
 	        System.out.println("TimerReceiver :: Start Service");
		} catch (InstanceAlreadyExistsException iae) {
			throw new ServiceException("Cannot start the TimerReceiver Service (Already Exists)" , iae);
		} catch (MBeanRegistrationException mbre) {
			throw new ServiceException("Cannot start the TimerReceiver Service (MBean registration error)" , mbre);
		} catch (NotCompliantMBeanException ncmbe) {
			throw new ServiceException("Cannot start the TimerReceiver Service (MBean Not compliant error)" , ncmbe);
		} catch (MalformedObjectNameException mone) {
			throw new ServiceException("Cannot start the TimerReceiver Service (ObjectName Malformed)" , mone);
		}
	}

	public void handleNotification(Notification n, Object handback) {
		BnProjectLocal mProject=null;
		BonitaProjectValue model;
		BonitaNodeValue node;
		Collection hooks;
		
		String type=(String)n.getType();
		String projectName=type.substring(type.indexOf(":")+1);
		projectName = projectName.substring(0,projectName.indexOf(":"));
		String nodeName=type.substring(0,type.indexOf(":"));				
		try{
			BnProjectLocalHome pHome = BnProjectUtil.getLocalHome();
			mProject=pHome.findByName(projectName);			
		
		}catch(Exception ple){ple.printStackTrace();}
		
		try {
			model = mProject.getModel();
			node = BonitaServiceValue.getNodeFromCache(model,nodeName);
			
			
			if (nodeName.matches(".*_instance.*"))
				nodeName=nodeName.substring(0,nodeName.indexOf("_instance"));

			BonitaNodeValue nodeLocal;
			nodeLocal = BonitaServiceValue.getNodeFromCache(model,nodeName);							
					
			hooks = nodeLocal.getHooks();
			for (Iterator i = hooks.iterator(); i.hasNext();) {
				BonitaHookValue hk = (BonitaHookValue) i.next();
				if (hk.getEvent().equalsIgnoreCase(hero.interfaces.Constants.Nd.ONDEADLINE)) {
					hero.hook.Hook h = hero.hook.Hook.make(hk.getName(), hero.interfaces.Constants.Nd.ONDEADLINE, hk.getType());
					//h.execute(this, hero.interfaces.Constants.Nd.ONDEADLINE, node);
				}
			}
						
		}catch(HeroException hh){hh.printStackTrace();};				
	}
	
	public void doStop() throws ServiceException {
		if (mbeanServer != null) {
			try {
				mbeanServer.unregisterMBean(new ObjectName(SERVICE_NAME));
			} catch (Exception e) {
				throw new ServiceException("Cannot stop the TimerReceiver Service (JMX).", e);
			}
		}
		System.out.println("TimerReceiver :: Stop Service");
	}
}
