/*
 * Created on 23 avr. 2004 by the Message Center Team
 *
 */
package mc.formgenerator.servlets;

import mc.formgenerator.servlets.util.*;

import hero.util.HeroException;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URI;
import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import hero.interfaces.*;

import javax.ejb.CreateException;
import javax.naming.NamingException;
import javax.security.auth.login.LoginException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import mc.formgenerator.bonita.BonitaActivityAdapter;
import mc.formgenerator.bonita.BonitaActivityExecutor;
import mc.formgenerator.bonita.BonitaProjectAdapter;
import mc.formgenerator.bonita.BonitaProjectExecutor;
import mc.formgenerator.bonita.DocumentParser;
import mc.formgenerator.ggf.GGF;

import org.apache.log4j.Category;
import org.chiba.xml.xforms.config.Config;
import org.chiba.xml.xforms.exception.XFormsException;
import org.chiba.adapter.ChibaAdapter;
import org.chiba.adapter.web.ServletAdapter;

/**
 * Servlet that will init and launch the form generator
 * @author sempereb
 */
public class ServletBonitaForm extends HttpServlet {

	//ServletAdapter instance
	private ServletAdapter servletAdapter = null;
	
	//GGF instance
	private GGF ggf = null;
	
	//Allow us to know if we are working with an activity or a project
	private Boolean isActivity; 
	
	//Name of the project we are working with
	private String projectName = null;

	//Name of the activity we are working with
	private String activityName = null;
		
	//Application mode consumer or exploitant
	private String mode = null;	
	
    //init-params
    private static Category cat = Category.getInstance(ServletView.class);
    private static final String FORM_PARAM_NAME = "form";
    private static final String XSL_PARAM_NAME = "xslt";
    private static final String CSS_PARAM_NAME = "css";
    private static final String ACTIONURL_PARAM_NAME = "action_url";	

    /*
     * It is not thread safe to modify these variables once the
     * init(ServletConfig) method has been called
     */
    // the absolute path to the Chiba config-file
    private String configPath = null;

    // the rootdir of this app; forms + documents fill be searched under this root
    private String contextRoot = null;

    // where uploaded files are stored
    private String uploadDir = null;

    private String stylesPath = null;
 
    
    /**
     * Returns a short description of the servlet.
     *
     * @return - Returns a short description of the servlet.
     */
    public String getServletInfo() {
        return "Servlet Controller for Chiba Bonita's XForms Processor";
    }

    /**
     * Destroys the servlet.
     */
    public void destroy() {
    }
    
    
	/**
	 * Start the server side interpretation of the xforms document
	 * @param document the xforms document we have to interprete
	 * @param writer the servlet's writer
	 */
	private void xformProcessing(Document instance, PrintWriter writer, HttpServletRequest request, String actionURL, HttpSession session) 
		throws 	TransformerConfigurationException, XFormsException,
		FileNotFoundException, TransformerException, IOException{
					
		//xmlBase needed by Chiba
		String xmlBase = request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort()+ request.getContextPath();
        //GGF launching
		Document formDocument = this.ggf.runGGF(instance,xmlBase);
		
		//Chiba launching
		//Build HTML page according to an activity or a project we are working with
		try{
	        String xslFile = request.getParameter(XSL_PARAM_NAME);
	        String css = request.getParameter(CSS_PARAM_NAME);			
			this.servletAdapter = setupServletAdapter(actionURL, session, formDocument, xslFile, css);
	        updateContext(request, session, instance);			
			
	        //add all request params that are not used by this servlet to the context map in ChibaBean
	        storeContextParams(request);
	        this.servletAdapter.init();
	        this.servletAdapter.executeHandler();
	        this.servletAdapter.buildUI(writer);
		} catch (Exception e) {
			//Error
			e.printStackTrace();
		}
	}
	
	/**
	 * Action on doGet
	 * @param writer Response's writer
	 * @param request Request access
	 * @param response servlet response
	 * @throws TransformerConfigurationException
	 * @throws FileNotFoundException
	 * @throws XFormsException
	 * @throws TransformerException
	 * @throws IOException
	 */
	private void writingUI(PrintWriter writer, HttpServletRequest request, HttpServletResponse response, String actionURL, HttpSession session) 
		throws 	LoginException, TransformerConfigurationException, 
				FileNotFoundException, TransformerException, 
				IOException, Exception{
		   	
		ProjectSessionHome projecth = null;

		ProjectSession project = null;
		
		//Document for server side interpretation
		Document instance = null;

		//We must get bonita project name
		this.projectName = request.getParameter("projectName");
		
		//We must get bonita activity name eventually
		this.activityName = request.getParameter("activityName");
	
		//Mode consumer or exploitant
		this.mode = request.getParameter("mode");
		
		//Start or stop the activity
		String action = request.getParameter("action");
		
		//Project name parameter is present
		if(this.projectName != null){
			
			//We are working with an activity
			if(this.activityName != null){
			
			    if (action != null){
				    
			        if(action.equals("stop")){
				        
						//Creation of executor
						BonitaActivityExecutor bonitaActivityExecutor = new BonitaActivityExecutor(); 
						
						//Terminate activity
						bonitaActivityExecutor.endActivity(this.activityName, this.projectName);
						
				        //Redirection to work list
						response.sendRedirect("/formgenerator/ServletWorkList");
						
						return;	
			        }
				    else{			    
						this.isActivity = new Boolean(true);
						
						//Creation of adapter
						BonitaActivityAdapter bonitaActivityAdapter = new BonitaActivityAdapter(); 
						
						//Getting process informations via the adapter
						instance = bonitaActivityAdapter.getActivityData(this.projectName, activityName);
				    }			        
			    }
			    else{			    
					this.isActivity = new Boolean(true);
					
					//Creation of adapter
					BonitaActivityAdapter bonitaActivityAdapter = new BonitaActivityAdapter(); 
					
					//Getting process informations via the adapter
					instance = bonitaActivityAdapter.getActivityData(this.projectName, activityName);
			    }
			}
			else{	
					
				//We are working with a project
				this.isActivity = new Boolean(false);

				
				//**************************************************************************
				//					INSTANCIATE THE PROCESS
				//**************************************************************************

				//Getting local home by using JNDI
				projecth = ProjectSessionUtil.getHome();

				//Creation of the project
				project = projecth.create();

				//Project instanciation
				this.projectName=project.instantiateProject(this.projectName);
				
				//Creation of adapter
				BonitaProjectAdapter bonitaProjectAdapter = new BonitaProjectAdapter(); 
					
				//Getting process informations via the adapter
				instance = bonitaProjectAdapter.getProjectData(this.projectName);
			}		
			
			DocumentParser parser = new DocumentParser();
			
			//If the document has properties
			if(parser.hasProperties(instance)){									
				//Finally starting of interpretation
				this.xformProcessing(instance, writer, request, actionURL, session);
			}
			else{
				//Call executor
				this.callExecutor(instance);
				if (request.getParameter("forward")!=null)
				{
					response.sendRedirect(request.getParameter("forward"));
			        }
				else
				{
	  				//Page redirection
					if(this.isActivity.booleanValue() == true){
					    
					    if(this.mode.equals("consumer"))
					        //Redirection to todo list
							response.sendRedirect("/formgenerator/ServletActivityToDoList?projectName=" + projectName);
					    else
					        //Redirection to work list
							response.sendRedirect("/formgenerator/ServletWorkList");
					}
					else{
					    if(this.mode.equals("consumer"))
					        //Redirection to instances projets
					        response.sendRedirect("/formgenerator/ServletInstancesProjects");
					    else
					        //Redirection to work list
							response.sendRedirect("/formgenerator/ServletWorkList");				        
					}
			        }
			}
		}
		else
			//Message to user
			(new MessageServlet()).writeMessage(writer, "PARAMETER FOR projectName IS MISSING CANNOT PERFORM OPERATION");
	}
	
	
	
	
	/**
	 * Call the correct executor
	 * @param filledDocument document that will be use by the executor
	 * @param mode the application mode: consumer or exploitant
	 */
	private void callExecutor(Document filledDocument) 
		throws 	MalformedURLException, SAXException, IOException, 
				RemoteException, NamingException, CreateException, HeroException{
			
		if(this.isActivity.booleanValue() == true){
			
			//Creation of executor
			BonitaActivityExecutor bonitaActivityExecutor = new BonitaActivityExecutor(); 
				
			//Terminate the activity
			bonitaActivityExecutor.actOnActivity(filledDocument, this.mode);	
		}
		else{

			//Creation of executor
			BonitaProjectExecutor bonitaProjectExecutor = new BonitaProjectExecutor(); 
				
			//Instanciate project
			bonitaProjectExecutor.startBonitaProject(filledDocument);
		}		
	}
	
	
	
		
    /**
     * Initializes the servlet.
     *
     * @param config - the ServletConfig object
     * @throws javax.servlet.ServletException
     */
    public void init(ServletConfig config) throws ServletException {
        super.init(config);

        cat.info("--------------- initing ChibaView... ---------------");
        //read some params from web-inf
        contextRoot = getServletConfig().getServletContext().getRealPath("");
        if (contextRoot == null)
            contextRoot = getServletConfig().getServletContext().getRealPath(".");

        //get the relative path to the chiba config-file
        String path = getServletConfig().getInitParameter("chiba.config");

        //get the real path for the config-file
        if (path != null) {
            configPath = getServletConfig().getServletContext().getRealPath(path);
        }
        
        //get the path for the stylesheets
        //path = getServletConfig().getServletContext().getInitParameter("chiba.xforms.stylesPath");
        path = getServletConfig().getInitParameter("chiba.xforms.stylesPath");

        //get the real path for the stylesheets and configure a new StylesheetLoader with it
        if (path != null) {
            stylesPath = getServletConfig().getServletContext().getRealPath(path);
            cat.info("stylesPath: " + stylesPath);
        }

        //uploadDir = contextRoot	+ "/" + getServletConfig().getServletContext().getInitParameter("chiba.upload");
        //uploadDir = getServletConfig().getServletContext().getInitParameter("chiba.upload");
        uploadDir = getServletConfig().getInitParameter("chiba.upload");
        //Security constraint
        if (uploadDir != null) {
            if (uploadDir.toUpperCase().indexOf("WEB-INF") >= 0) {
                throw new ServletException("Chiba security constraint: uploadDir '" + uploadDir + "' not allowed");
            }
        }
    }
	
	
	
	
	/**
	 * Servlet reaction on get method:
	 * Display the form generated for filling a valid XML file
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
		
		this.servletAdapter = null;

		//Creating a session
		HttpSession session = request.getSession(true);

		// get the user language
		String userLanguage = request.getHeader("Accept-Language");

		//write the 'forward' parameter in the session
		if (request.getParameter("forward")!=null)
		    session.setAttribute("forward",request.getParameter("forward"));
		else
		    session.removeAttribute("forward");
				
		try {
			//Content type
			response.setContentType("text/html");
				
			//Getting the writer
			PrintWriter out = response.getWriter();

            // build actionURL where forms are submitted to
            String actionURL = getActionURL(request, response);
            
		    //instanciate the generator with a transformation file
			if (request.getParameter("transform")!=null){
				this.ggf = new GGF(this.contextRoot, actionURL,request.getParameter("transform"), userLanguage);
			}
			//instanciate with the default xsl file
			else this.ggf = new GGF(this.contextRoot, actionURL, userLanguage);
						
			
			//Build HTML page according to an activity or a project we are working with
			writingUI(out, request, response, actionURL, session);
			
			//Stores the GGF in the session
			session.setAttribute("formgenerator.GGF", this.ggf);
			
			//Stores the state of isActivity attribute in the session
			session.setAttribute("formgenerator.isActivity", this.isActivity);
			
			//Stores the projectname in the session
			session.setAttribute("formgenerator.projectName", this.projectName);
			
			//Stores the activity name in the session
			session.setAttribute("formgenerator.activityName", this.activityName);
			
			//Stores the application mode in the session
			session.setAttribute("formgenerator.mode", this.mode);
		
			//Stores the servletAdapter
			session.setAttribute("formgenerator.sAdapter", this.servletAdapter);
			
			//Closing writer
			out.close();
			
		} catch (Exception e) {
			//Error
			e.printStackTrace();
		}
	}
	
    /**
     * this method is responsible for passing all context information needed by the Adapter and Processor from
     * ServletRequest to ChibaContext.
     *
     * @param servletAdapter the ChibaAdapter to use
     * @param request        the ServletRequest
     * @param session        the ServletSession
     */
    protected void updateContext(HttpServletRequest request, HttpSession session,Document formDocument) {
    	this.servletAdapter.setContextProperty(ServletAdapter.USERAGENT, request.getHeader("User-Agent"));
    	this.servletAdapter.setContextProperty(ServletAdapter.HTTP_SERVLET_REQUEST, request);
    	this.servletAdapter.setContextProperty(ServletAdapter.HTTP_SESSION_OBJECT, session);
    	this.servletAdapter.setContextProperty("instance",formDocument);
    }	
	
		
	/**
	 * Servlet reaction on post method:
	 * Submit information from user
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response)	throws ServletException, IOException
	{	
		//Getting the current session
		HttpSession session = request.getSession(true);
		this.servletAdapter = null;
		
		//Getting the writer
		PrintWriter out = response.getWriter();
		
		try {
			//Getting servletAdapter from the session
			this.servletAdapter = (ServletAdapter)session.getAttribute("formgenerator.sAdapter");				

			//Getting GGF from the session
			this.ggf = (GGF)session.getAttribute("formgenerator.GGF");
			//Reset the GGF in the session
			session.setAttribute("formgenerator.GGF", this.ggf);		
			
			//Getting isActivity from the session
			this.isActivity = (Boolean)session.getAttribute("formgenerator.isActivity");
			//Set isActivity in the session
			session.setAttribute("formgenerator.isActivity", this.isActivity);		
			
			//Getting projectName from the session
			this.projectName = (String)session.getAttribute("formgenerator.projectName");		
			//Set the projectname in the session
			session.setAttribute("formgenerator.projectName", this.projectName);
			
			//Getting activityName from the session
			this.activityName = (String)session.getAttribute("formgenerator.activityName");	
			//Set the activityname in the session
			session.setAttribute("formgenerator.activityName", this.activityName);
				
			//Getting mode from the session
			this.mode = (String)session.getAttribute("formgenerator.mode");
			
			
			if ((this.servletAdapter == null)||(this.ggf == null)||(this.isActivity == null)||(this.projectName == null)){
				//Exception
				throw new IOException("Invalid session - Cannot retrieve servletAdapter or ggf or isActivity or projectName in the session");
			}
					
			updateContextPost(request, session);
            
            this.servletAdapter.executeHandler();				
            		
            //get instance filled
			Document filledDocument = (Document)this.servletAdapter.getChibaBean().getContainer().getModel("model").getInstance("instance").getInstanceDocument();
			
            if (filledDocument == null) {
                throw new ServletException(Config.getInstance().getErrorMessage("session-invalid"));
            }
            

			
			//Set content type
			response.setContentType("text/html");
			
			//Getting filled document
			//Document filledDocument = servletAdapter.getChibaBean().getContainer().getDefaultModel().getInstanceDocument("instance");
			
			
			//Calling the executor
			this.callExecutor(filledDocument);
			
			//Page redirection
		
			//use the forward parameter form the session in order to redirect the output
			String forward = (String)session.getAttribute("forward");
			if ((forward!=null)&&!forward.equals(""))
			    response.sendRedirect(forward);
			
			//use default redirection
			else
				if(this.isActivity.booleanValue() == true){
				    
				    if(this.mode.equals("consumer"))
				        //Redirection to todo list
						response.sendRedirect("/formgenerator/ServletActivityToDoList?projectName=" + projectName);
				    else
				        //Redirection to work list
						response.sendRedirect("/formgenerator/ServletWorkList");
				}
				else{
				    if(this.mode.equals("consumer"))
				        //Redirection to instances projets
				        response.sendRedirect("/formgenerator/ServletInstancesProjects");
				    else
				        //Redirection to work list
						response.sendRedirect("/formgenerator/ServletWorkList");				        
				}
				
				
			//Render result to output
			//this.chiba.runChiba(out);
			this.servletAdapter.buildUI(out);
			
			//Close output stream
			out.close();
			
		} catch (Exception e) {
			//Error
			e.printStackTrace();
		}	
	}
  
	
    /**
     * this method is responsible for passing all context information needed by the Adapter and Processor from
     * ServletRequest to ChibaContext.
     *
     * @param servletAdapter the ChibaAdapter to use
     * @param request        the ServletRequest
     * @param session        the ServletSession
     */
    protected void updateContextPost(HttpServletRequest request, HttpSession session) {
    	this.servletAdapter.setContextProperty(ServletAdapter.USERAGENT, request.getHeader("User-Agent"));
    	this.servletAdapter.setContextProperty(ServletAdapter.HTTP_SERVLET_REQUEST, request);
    	this.servletAdapter.setContextProperty(ServletAdapter.HTTP_SESSION_OBJECT, session);
    }	
	

    /**
     * creates and configures the ServletAdapter which does the actual request processing.
     *
     * @param actionURL - the URL to submit to
     * @param session   - the Servlet session
     * @param formPath  - the relative location where forms are stored
     * @param xslFile   - the xsl file to use for transform
     * @param cssFile   - the CSS file to use for styling the output
     * @return ServletAdapter
     */
    private ServletAdapter setupServletAdapter(String actionURL,
                                               HttpSession session,
                                               Document formDocument,
                                               String xslFile,
                                               String cssFile) {
        //setup and configure the adapter
        ServletAdapter aAdapter = new ServletAdapter();
        aAdapter.setContextRoot(contextRoot);
        if ((configPath != null) && !(configPath.equals(""))) {
            aAdapter.setConfigPath(configPath);
        }
        aAdapter.setFormDocument(formDocument);
        aAdapter.setStylesheetPath(stylesPath);
        aAdapter.setActionUrl(actionURL);
        aAdapter.setUploadDir(uploadDir);

        if (xslFile != null) {
            aAdapter.setStylesheet(xslFile);
            if (cat.isDebugEnabled()) {
                cat.debug("using xsl stylesheet: " + xslFile);
            }
        }
        if (cssFile != null) {
            aAdapter.setCSS(cssFile);
            if (cat.isDebugEnabled()) {
                cat.debug("using css stylesheet: " + cssFile);
            }
        }

        Map servletMap = new HashMap();
        servletMap.put(ChibaAdapter.SESSION_ID, session.getId());
        aAdapter.setContextProperty(ChibaAdapter.SUBMISSION_RESPONSE, servletMap);

        return aAdapter;
    }	
    
    private void storeContextParams(HttpServletRequest request) {
        Enumeration params = request.getParameterNames();
        String s;
        while (params.hasMoreElements()) {
            s = (String) params.nextElement();
            //store all request-params we don't use in the context map of ChibaBean
            if (!(s.equals(FORM_PARAM_NAME) || s.equals(XSL_PARAM_NAME) || s.equals(CSS_PARAM_NAME) || s.equals(ACTIONURL_PARAM_NAME))) {
                String value = request.getParameter(s);
                this.servletAdapter.setContextProperty(s, value);
                if (cat.isDebugEnabled()) {
                    cat.debug("added request param '" + s + "' added to context");
                }
            }
        }
    }     


	/**
	 * Return the action URL with jsession
	 * @param request : The request from user
	 * @param response : The response to its request
	 * @return a String object that represents the action URL
	 */
    private String getActionURL(HttpServletRequest request, HttpServletResponse response) {
        String defaultActionURL =
                request.getScheme()
                + "://"
                + request.getServerName()
                + ":"
                + request.getServerPort()
                + request.getContextPath()
                + request.getServletPath();
        String encodedDefaultActionURL = response.encodeURL(defaultActionURL);
        int sessIdx = encodedDefaultActionURL.indexOf(";jsession");
        String sessionId = null;
        if (sessIdx > -1) {
            sessionId = encodedDefaultActionURL.substring(sessIdx);
        }
        String actionURL = request.getParameter(ACTIONURL_PARAM_NAME);
        if (null == actionURL) {
            actionURL = encodedDefaultActionURL;
        } else if (null != sessionId) {
            actionURL += sessionId;
        }

        cat.info("actionURL: " + actionURL);
        // encode the URL to allow for session id rewriting
        actionURL = response.encodeURL(actionURL);
        return actionURL;
    }
}
